# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator/create_python_api.py script.
"""Operations for working with string Tensors.
"""

from __future__ import print_function as _print_function

import sys as _sys

from tensorflow.python.ops.gen_string_ops import as_string
from tensorflow.python.ops.gen_string_ops import string_join as join
from tensorflow.python.ops.gen_string_ops import string_lower as lower
from tensorflow.python.ops.gen_string_ops import string_strip as strip
from tensorflow.python.ops.gen_string_ops import string_to_hash_bucket_fast as to_hash_bucket_fast
from tensorflow.python.ops.gen_string_ops import string_to_hash_bucket_strong as to_hash_bucket_strong
from tensorflow.python.ops.gen_string_ops import string_upper as upper
from tensorflow.python.ops.gen_string_ops import unicode_script
from tensorflow.python.ops.gen_string_ops import unicode_transcode
from tensorflow.python.ops.gen_string_ops import unsorted_segment_join
from tensorflow.python.ops.ragged.ragged_string_ops import ngrams
from tensorflow.python.ops.ragged.ragged_string_ops import string_bytes_split as bytes_split
from tensorflow.python.ops.ragged.ragged_string_ops import strings_split_v1 as split
from tensorflow.python.ops.ragged.ragged_string_ops import unicode_decode
from tensorflow.python.ops.ragged.ragged_string_ops import unicode_decode_with_offsets
from tensorflow.python.ops.ragged.ragged_string_ops import unicode_encode
from tensorflow.python.ops.ragged.ragged_string_ops import unicode_split
from tensorflow.python.ops.ragged.ragged_string_ops import unicode_split_with_offsets
from tensorflow.python.ops.string_ops import reduce_join
from tensorflow.python.ops.string_ops import regex_full_match
from tensorflow.python.ops.string_ops import regex_replace
from tensorflow.python.ops.string_ops import string_format as format
from tensorflow.python.ops.string_ops import string_length as length
from tensorflow.python.ops.string_ops import string_to_hash_bucket_v1 as to_hash_bucket
from tensorflow.python.ops.string_ops import string_to_number_v1 as to_number
from tensorflow.python.ops.string_ops import substr

del _print_function

from tensorflow.python.util import module_wrapper as _module_wrapper

if not isinstance(_sys.modules[__name__], _module_wrapper.TFModuleWrapper):
  _sys.modules[__name__] = _module_wrapper.TFModuleWrapper(
      _sys.modules[__name__], "strings", public_apis=None, deprecation=True,
      has_lite=False)
