# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator/create_python_api.py script.
"""Public API for tf.layers namespace.
"""

from __future__ import print_function as _print_function

import sys as _sys

from tensorflow._api.v1.compat.v1.layers import experimental
from tensorflow.python.keras.engine.input_spec import InputSpec
from tensorflow.python.layers.base import Layer
from tensorflow.python.layers.convolutional import Conv1D
from tensorflow.python.layers.convolutional import Conv2D
from tensorflow.python.layers.convolutional import Conv2DTranspose
from tensorflow.python.layers.convolutional import Conv3D
from tensorflow.python.layers.convolutional import Conv3DTranspose
from tensorflow.python.layers.convolutional import SeparableConv1D
from tensorflow.python.layers.convolutional import SeparableConv2D
from tensorflow.python.layers.convolutional import conv1d
from tensorflow.python.layers.convolutional import conv2d
from tensorflow.python.layers.convolutional import conv2d_transpose
from tensorflow.python.layers.convolutional import conv3d
from tensorflow.python.layers.convolutional import conv3d_transpose
from tensorflow.python.layers.convolutional import separable_conv1d
from tensorflow.python.layers.convolutional import separable_conv2d
from tensorflow.python.layers.core import Dense
from tensorflow.python.layers.core import Dropout
from tensorflow.python.layers.core import Flatten
from tensorflow.python.layers.core import dense
from tensorflow.python.layers.core import dropout
from tensorflow.python.layers.core import flatten
from tensorflow.python.layers.normalization import BatchNorm as BatchNormalization
from tensorflow.python.layers.normalization import batch_norm as batch_normalization
from tensorflow.python.layers.pooling import AveragePooling1D
from tensorflow.python.layers.pooling import AveragePooling2D
from tensorflow.python.layers.pooling import AveragePooling3D
from tensorflow.python.layers.pooling import MaxPool2D as MaxPooling2D
from tensorflow.python.layers.pooling import MaxPooling1D
from tensorflow.python.layers.pooling import MaxPooling3D
from tensorflow.python.layers.pooling import average_pooling1d
from tensorflow.python.layers.pooling import average_pooling2d
from tensorflow.python.layers.pooling import average_pooling3d
from tensorflow.python.layers.pooling import max_pool2d as max_pooling2d
from tensorflow.python.layers.pooling import max_pooling1d
from tensorflow.python.layers.pooling import max_pooling3d

del _print_function

from tensorflow.python.util import module_wrapper as _module_wrapper

if not isinstance(_sys.modules[__name__], _module_wrapper.TFModuleWrapper):
  _sys.modules[__name__] = _module_wrapper.TFModuleWrapper(
      _sys.modules[__name__], "compat.v1.layers", public_apis=None, deprecation=False,
      has_lite=False)
