# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator/create_python_api.py script.
"""Operations for manipulating the binary representations of integers.
"""

from __future__ import print_function as _print_function

import sys as _sys

from tensorflow.python.ops.gen_bitwise_ops import bitwise_and
from tensorflow.python.ops.gen_bitwise_ops import bitwise_or
from tensorflow.python.ops.gen_bitwise_ops import bitwise_xor
from tensorflow.python.ops.gen_bitwise_ops import invert
from tensorflow.python.ops.gen_bitwise_ops import left_shift
from tensorflow.python.ops.gen_bitwise_ops import right_shift

del _print_function

from tensorflow.python.util import module_wrapper as _module_wrapper

if not isinstance(_sys.modules[__name__], _module_wrapper.TFModuleWrapper):
  _sys.modules[__name__] = _module_wrapper.TFModuleWrapper(
      _sys.modules[__name__], "bitwise", public_apis=None, deprecation=True,
      has_lite=False)
