# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator/create_python_api.py script.
"""Keras initializer serialization / deserialization.

"""

from __future__ import print_function as _print_function

import sys as _sys

from tensorflow.python.keras.initializers import deserialize
from tensorflow.python.keras.initializers import get
from tensorflow.python.keras.initializers import serialize
from tensorflow.python.ops.init_ops_v2 import Constant
from tensorflow.python.ops.init_ops_v2 import Constant as constant
from tensorflow.python.ops.init_ops_v2 import GlorotNormal
from tensorflow.python.ops.init_ops_v2 import GlorotNormal as glorot_normal
from tensorflow.python.ops.init_ops_v2 import GlorotUniform
from tensorflow.python.ops.init_ops_v2 import GlorotUniform as glorot_uniform
from tensorflow.python.ops.init_ops_v2 import Identity
from tensorflow.python.ops.init_ops_v2 import Identity as identity
from tensorflow.python.ops.init_ops_v2 import Initializer
from tensorflow.python.ops.init_ops_v2 import Ones
from tensorflow.python.ops.init_ops_v2 import Ones as ones
from tensorflow.python.ops.init_ops_v2 import Orthogonal
from tensorflow.python.ops.init_ops_v2 import Orthogonal as orthogonal
from tensorflow.python.ops.init_ops_v2 import RandomNormal
from tensorflow.python.ops.init_ops_v2 import RandomUniform
from tensorflow.python.ops.init_ops_v2 import TruncatedNormal
from tensorflow.python.ops.init_ops_v2 import VarianceScaling
from tensorflow.python.ops.init_ops_v2 import Zeros
from tensorflow.python.ops.init_ops_v2 import Zeros as zeros
from tensorflow.python.ops.init_ops_v2 import he_normal
from tensorflow.python.ops.init_ops_v2 import he_uniform
from tensorflow.python.ops.init_ops_v2 import lecun_normal
from tensorflow.python.ops.init_ops_v2 import lecun_uniform

del _print_function

from tensorflow.python.util import module_wrapper as _module_wrapper

if not isinstance(_sys.modules[__name__], _module_wrapper.TFModuleWrapper):
  _sys.modules[__name__] = _module_wrapper.TFModuleWrapper(
      _sys.modules[__name__], "keras.initializers", public_apis=None, deprecation=False,
      has_lite=False)
